import { Body, Controller, Get, Inject, Post } from '@midwayjs/core';
import { Context } from '@midwayjs/koa';
import { BizCode } from '../biz/code';
import {
  adminSign,
  adminSignExpired,
  webSign,
  webSignExpired,
} from '../config/base.config';
import { UserAdminAuthDTO, UserWebAuthDTO } from '../dto/user.dto';
import { XCodeService } from '../service/xcode.service';
import { UserService } from '../service/user.service';
import { createToken, decodeToken } from '../util/encrypt';
import { SmsService } from '../service/sms.service';
import { SmsDTO } from '../dto/sms.dto';
import { RedisService } from '@midwayjs/redis';

@Controller('/user')
export class UserController {
  @Inject()
  ctx: Context;

  @Inject()
  userService: UserService;

  @Inject()
  xcodeService: XCodeService;

  @Inject()
  smsService: SmsService;

  @Inject()
  redisService: RedisService;

  /**
   * 用户前台登录
   */
  @Post('/web/auth')
  async webAuth(@Body() params: UserWebAuthDTO) {
    try {
      const verifyCode = await this.redisService.get('' + params.user_login);
      if (!verifyCode) return { code: BizCode.ERROR, msg: '验证码无效' };
      // 查询用户是否存在
      const userExist = await this.userService.select(params);
      let payload = {};
      if (userExist?.id) {
        const { user_pass, ...rest } = userExist;
        payload = rest;
      } else {
        // 新用户注册
        const { user_pass, ...rest } = await this.userService.createUser(
          params
        );
        payload = rest;
      }
      const token = createToken({ ...payload, hasLogin: true });
      this.ctx.cookies.set(webSign, token, {
        expires: new Date(Date.now() + webSignExpired),
        httpOnly: false,
      });
      await this.redisService.del('' + params.user_login);
      return {
        code: BizCode.OK,
        msg: '欢迎来到 backset.cn',
        data: { ...payload },
      };
    } catch (error) {
      this.ctx.logger.error(error);
      return { code: BizCode.ERROR, msg: '[error] web/auth error' };
    }
  }

  /**
   * 管理员登录
   */
  @Post('/admin/auth')
  async AdminAuth(@Body() params: UserAdminAuthDTO) {
    const { username, password } = params;
    const token = createToken({ hasLogin: true });
    if (username === 'admin' && password === '123123') {
      this.ctx.cookies.set(adminSign, token, {
        expires: new Date(Date.now() + adminSignExpired),
        httpOnly: false,
      });
      return { code: BizCode.OK };
    } else {
      return { code: BizCode.ERROR, msg: '用户名密码错误' };
    }
  }

  @Get('/web/state')
  async state() {
    try {
      const token = this.ctx.cookies.get(webSign);
      const user = decodeToken(token);
      return { code: BizCode.OK, data: user };
    } catch (error) {
      this.ctx.logger.error(error);
      return { code: BizCode.ERROR, msg: '[error] /web/state error' };
    }
  }

  @Post('/web/sms')
  async verifyCode(@Body() params: SmsDTO) {
    try {
      const { phoneNumber: phoneNumbers } = params;
      const code = Math.floor(Math.random() * 9000 + 1000);
      await this.redisService.set('' + phoneNumbers, code, 'EX', 60);
      console.log('redis here');
      await this.smsService.send({ code, phoneNumbers });
      console.log('sms here');
      return { code: BizCode.OK };
    } catch (error) {
      console.log(error);
      this.ctx.logger.error(error);
      return { code: BizCode.ERROR, msg: '[error] /web/sms error' };
    }
  }
}
